/*
 * Copyright 2004-2009 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.api.dn;

import moazs.api.MoaException;

import org.w3c.dom.Element;

/**
 * This interface declares the methods to implement for handling both
 * MOA-ZS and ZUSE (delivery server) notifications.
 */
public interface DeliveryNotificationHandler {

  /**
   * Handles a delivery notification of a delivery server, which is is
   * signed by the delivery server and contains the delivery confirmation
   * signed by the user (RSa).
   * 
   * @param deliveryNotificationElement the XML element representing the
   * delivery notification.
   * @throws MoaException when an error occurs handling the delivery
   * notification.
   */
  void handleZUSEDeliveryNotification(Element deliveryNotificationElement) throws MoaException;
  
  /**
   * Handles a delivery notification of MOA-ZS, which represents the delivery
   * status (SUCCESS, ERROR). Success means the delivery
   * has been successfully delivery to the server, error means some error has
   * occured and the delivery could not be handed over to the server.
   * 
   * @param deliveryNotificationElement the XML element representing the
   * delivery notification.
   * @throws MoaException when an error occurs handling the delivery
   * notification.
   */
  void handleMZSDeliveryNotification(Element deliveryNotificationElement) throws MoaException;
}
