package moazs.api.dn.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import moazs.api.MoaException;
import moazs.api.dn.AbstractDeliveryNotificationHandler;
import moazs.util.MoaXMLUtils;

import org.apache.log4j.Logger;
import org.w3c.dom.Element;

public class FileDeliveryNotificationHandler extends
    AbstractDeliveryNotificationHandler {

  static private Logger logger = Logger.getLogger(FileDeliveryNotificationHandler.class);
  
  private File directory = null;
  
  public FileDeliveryNotificationHandler() {
    String directoryString = super.getPluginProperty("directory");
    if (directoryString == null || "".equals(directoryString)) {
      logger.warn("Init: No directory for delivery notification output files for plugin " + FileDeliveryNotificationHandler.class.getName() + " specified. Use the 'directory' property.");
    } else {
      File tmpFile = new File(directoryString);
      if (!tmpFile.isDirectory()) {
        logger.warn("Init: Configured directory for plugin " + FileDeliveryNotificationHandler.class.getName() + " is not a valid directory.");
      } else {
        this.directory = tmpFile;
      }
    }
    
  }
  
  public void handleZUSEDeliveryNotification(Element deliveryNotificationElement)
      throws MoaException {
    
    writeDeliveryNotification(deliveryNotificationElement, true);
  }
  
  public void handleMZSDeliveryNotification(Element deliveryNotificationElement)
    throws MoaException {
  
    writeDeliveryNotification(deliveryNotificationElement, false);
  }
  
  private void writeDeliveryNotification(Element deliveryNotificationElement, boolean zuse) throws MoaException {
    if (this.directory == null) {
      logger.warn("No directory for delivery notification output files for plugin " + FileDeliveryNotificationHandler.class.getName() + " specified. Use the 'directory' property.");
    } else {
      // Extract AppId Element
      //String appDeliveryId = MoaXPath.getFirstString(deliveryNotificationElement, "//zus:AppDeliveryID/text()");
      
      // create filename
      SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
      
      String fileName = null;
      if (zuse) {
        fileName = directory.getAbsolutePath() + "/" + sdf.format(new Date()) + ".zus.xml";
      } else {
        fileName = directory.getAbsolutePath() + "/" + sdf.format(new Date()) + ".mzs.xml";
      }
      
      File out = new File(fileName);
      if(!this.directory.canWrite()) {
        logger.warn("Cannot write delivery notification to file " + out.getAbsoluteFile() + ". Insufficient write rights.");
      } else {
        try {
          if (zuse) {
            logger.info("Writing ZUSE delivery notification to file " + out.getAbsoluteFile());
          } else {
            logger.info("Writing MZS delivery notification to file " + out.getAbsoluteFile());
          }
          FileOutputStream fos = new FileOutputStream(out);
          fos.write(MoaXMLUtils.serializeToByteArray(deliveryNotificationElement, true));
          fos.flush();
          fos.close();
        } catch(IOException e) {
          logger.warn("Cannot write delivery notification to file " + out.getAbsoluteFile() + ". I/O error: " + e.getMessage(), e);
        }
      }
    }  

  }

}
