package moazs.aspects;

import java.util.logging.FileHandler;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;


public aspect TimerAspect {

	//zeitmessung fuer void methoden,
	//allerdings keine methoden in aspects, modell, util	
    pointcut anyPublicVoidMethod(): 
        execution(public void *.*(..))
        && !within(moazs.aspects.*)
        && !within(moazs.modell.*)
        && !within(moazs.util.*);    	

	//zeitmessung fuer int methoden
    pointcut anyPublicIntMethod(): 
        execution(public int *.*(..))
        && !within(moazs.modell.*)
        && !within(moazs.util.*)
        && !within(moazs.aspects.*);
    
    //zeitmessung fuer methoden mit document
    pointcut anyDocumentMethod():
    	execution(public Document *.*(..))
    	&& !within(moazs.modell.*)
        && !within(moazs.util.*)
        && !within(moazs.aspects.*);
    
    static private Logger logger = Logger.getLogger(TimerAspect.class);
    
    TimerAspect() {
        FileHandler tfh = null;
        try {
            tfh = new FileHandler("%t/timing%u.log",1000000, 1);
            tfh.setFormatter(new MoaLogFormatter());
        } catch (SecurityException se) {
            se.printStackTrace();
        } catch (java.io.IOException ie) {
        	ie.printStackTrace();
        }
    }

    void around(): anyPublicVoidMethod() {
      long startTime = System.currentTimeMillis();
      try {
        proceed();
      } finally {
        long dt = System.currentTimeMillis() - startTime;
	    logger.info(thisJoinPoint.getSignature().getDeclaringType().getName() + " " + 
	            thisJoinPoint.getSignature().getName() +
	            " dauerte " + dt + " ms.");
      }
    }

    int around(): anyPublicIntMethod() {
      long startTime = System.currentTimeMillis();
      try {
        return proceed();
      } finally {
        long dt = System.currentTimeMillis() - startTime;
        logger.info(thisJoinPoint.getSignature().getDeclaringType().getName() + " " + 
	            thisJoinPoint.getSignature().getName() +
	            " dauerte " + dt + " ms.");
      }
    }

	Document around(): anyDocumentMethod() {
      long startTime = System.currentTimeMillis();
      try {
        return proceed();
      } finally {
        long dt = System.currentTimeMillis() - startTime;
        logger.info(thisJoinPoint.getSignature().getDeclaringType().getName() + " " + 
	            thisJoinPoint.getSignature().getName() +
	            " dauerte " + dt + " ms.");
      }
    }
	
  }


