/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.external;

import java.text.SimpleDateFormat;

import org.apache.log4j.Logger;

import moazs.Constants;
import moazs.api.MoaException;
import moazs.modell.Rcpt;
import moazs.szr.SZRClientException;
import moazs.util.MoaConfig;
import at.gv.bmi.szr.FremdBPKType;
import at.gv.bmi.szr.PersonInfoType;
import at.gv.bmi.szr.person.PersonNameType;
import at.gv.bmi.szr.person.PhysicalPersonType;

/**
* Klasse zum umrechnen von Bereichkennungen via SZR
*/

public class MoaSZR {

//    private String NS_SZR  = null;
//    private String NS_PERS = null;
//    private String NS_DSIG = null;
    private String BPK_TYPE = null;
    
    static private Logger logger = Logger.getLogger(MoaSZR.class);

//    private String endpoint = null;
//    private String pvpns = null;
//    private Properties pvpheader = null;

    private MoaConfig mcfg = null;
    
    /**
    * Standard Konstruktor
    */
    public MoaSZR() {
        mcfg = new MoaConfig();
        this.BPK_TYPE = mcfg.getBereichskennung();
    }

    /**
    * Methode die die BPK <em>bpk</em> vom Typ <em>type</em> in die ZustellBPK umrechnet
     * @param rcpt Empfaengerobjekt
     * @param type Typ der BPK
     * @param bpk bPK
     * @return String ZustellBPK
     * @throws MoaException 
     */
    public String transform(Rcpt rcpt, String type, String bpk) throws MoaException {
        /*String result = "";

        try {
            Document request = makeRequest(rcpt, type, bpk);
            Service service = ServiceFactory.newInstance().createService(new QName("TransformBPK"));
            Call call = (Call)service.createCall();

            for (Iterator i = pvpheader.keySet().iterator(); i.hasNext(); ) {
                String key = (String)i.next();
                String value = (String)pvpheader.get(key);
                call.addHeader(new SOAPHeaderElement( pvpns, key, value ));
            }
            SOAPBodyElement body = new SOAPBodyElement(request.getDocumentElement());
            SOAPBodyElement[] params = new SOAPBodyElement[] {body};

            call.setTargetEndpointAddress( this.endpoint );
            Vector responses = (Vector) call.invoke(params);

            SOAPBodyElement response = (SOAPBodyElement) responses.get(0);
            Document resdoc =  response.getAsDocument();
            
            result  = MoaXPath.getFirstString( resdoc.getDocumentElement(), "/szr:TransformBPKResponse/szr:TransformBPKReturn/szr:BPK/text()");

        } catch (java.rmi.RemoteException re) {
            logger.log(Level.SEVERE, "RemoteException", re);
        } catch (java.io.IOException ioe) {
            logger.log(Level.SEVERE, "IO Exception", ioe);
        } catch (javax.xml.rpc.ServiceException se) {
            logger.log(Level.SEVERE, "ServiceException", se);
        } catch (java.lang.Exception ex) {
            logger.log(Level.SEVERE, "Exception", ex);
        }
        
        return result;*/
      logger.debug("Starte bPK Transformation am Stammzahlenregister.");
      PersonInfoType personInfoType = new PersonInfoType();
      PhysicalPersonType physicalPerson = new PhysicalPersonType();
      personInfoType.setPerson(physicalPerson);
      PersonNameType name = new PersonNameType();
      physicalPerson.setName(name);
      name.setGivenName(rcpt.getGivenname());
      name.setFamilyName(rcpt.getFamilyname());
      physicalPerson.setDateOfBirth(new SimpleDateFormat("yyyy-MM-dd").format(rcpt.getDateofbirth()));
      
      try {
        FremdBPKType fremdBPK = mcfg.getSZRClient().processTranslateBPK(personInfoType, bpk, type, BPK_TYPE, mcfg.getVKZ(), "MOA-ZS Zustellung");
        return fremdBPK.getFremdBPK();
      } catch (SZRClientException e) {
        // unclassified SZR error
        if (e.getSZRErrorCode() == -1) {
          throw new MoaException(e.getMessage(), Constants.ERROR_SZR_UNCLASSIFIED);
        } else {
          throw new MoaException(e.getMessage(), Constants.ERROR_SZR_UNCLASSIFIED + e.getSZRErrorCode());
        }
      }
    }

    /**
    * Methode die Den SoapRequest aufbaut
     * @param rcpt Empfaengerobjekt
     * @param type Typ der bPK
     * @param bpk aktuelle bPK
     * @return DOM-Document
     */
    /*public Document makeRequest(Rcpt rcpt, String type, String bpk) throws MoaException {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document result = db.getDOMImplementation().createDocument(NS_SZR, "szr:TransformBPK", null);
            Element root = result.getDocumentElement();
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:szr", NS_SZR); 
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:pers", NS_PERS); 
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:dsig", NS_DSIG); 

            Element personInfo  = result.createElementNS(NS_SZR, "szr:PersonInfo");
            Element person      = result.createElementNS(NS_SZR, "szr:Person");
            Element personName  = result.createElementNS(NS_PERS, "pers:Name");
            Element givenName   = result.createElementNS(NS_PERS, "pers:GivenName");
            givenName.appendChild( result.createTextNode( rcpt.getGivenname() ));
            Element familyName  = result.createElementNS(NS_PERS, "pers:FamilyName");
            familyName.appendChild( result.createTextNode( rcpt.getFamilyname() ));
            Element dateOfBirth = result.createElementNS(NS_PERS, "pers:DateOfBirth");
            dateOfBirth.appendChild( result.createTextNode( new SimpleDateFormat("yyyy-MM-dd").format(rcpt.getDateofbirth()) ));
            
            personInfo.appendChild(person);
            person.appendChild(personName);
            personName.appendChild(givenName);
            personName.appendChild(familyName);
            person.appendChild(dateOfBirth);
            root.appendChild(personInfo); 
            Element inputBPK      = result.createElementNS(NS_SZR, "szr:InputBPK");
            inputBPK.appendChild( result.createTextNode( bpk ));
            Element inputBereich  = result.createElementNS(NS_SZR, "szr:InputBereichsKennung");
            inputBereich.appendChild( result.createTextNode( type ));

            Element outputBereich = result.createElementNS(NS_SZR, "szr:OutputBereich");
            Element outputBereichsKennung = result.createElementNS(NS_SZR, "szr:Bereichskennung");
//            outputBereichsKennung.appendChild( result.createTextNode( "urn:publicid:gv.at:cdid+ZUST" ));
            outputBereichsKennung.appendChild( result.createTextNode( BPK_TYPE ));

            Element outputKey     = result.createElementNS(NS_SZR, "szr:PublicKey");

            Element rsakey        = result.createElementNS(NS_DSIG, "dsig:RSAKeyValue");
            outputKey.appendChild(rsakey); 
            
            Element modulus       = result.createElementNS(NS_DSIG, "dsig:Modulus");
            rsakey.appendChild(modulus);
            RSAPublicKey key = getZusePublicKey();
            modulus.appendChild( result.createTextNode( new String( Base64.encode(key.getModulus().toByteArray()))));
            Element exponent      = result.createElementNS(NS_DSIG, "dsig:Exponent");
            exponent.appendChild( result.createTextNode( new String( Base64.encode(key.getPublicExponent().toByteArray()))));

            rsakey.appendChild(exponent);

            root.appendChild( inputBPK );
            root.appendChild( inputBereich );
            root.appendChild( outputBereich );
            outputBereich.appendChild( outputBereichsKennung );
            outputBereich.appendChild( outputKey );

            return result;
        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            logger.log(Level.SEVERE, "ParserConfigException", pce);
            throw new MoaException(pce);
        }
    }*/
    
    /**
    * Den PublicKey des ZustellServer aus dem KonfigurationsFile auslesen.
     * @return RSAPublicKey
     */
    /*public RSAPublicKey getZusePublicKey() throws MoaException {
        try {
            MoaConfig mcfg = new MoaConfig();
            String pub64 = mcfg.getProperty("publicKey", "moaszr");
            byte[] bin = Base64.decode(pub64);
            X509EncodedKeySpec keySpec = new X509EncodedKeySpec(bin);
            KeyFactory factory = KeyFactory.getInstance("RSA");
            RSAPublicKey key = (RSAPublicKey)factory.generatePublic(keySpec);

            return key;
        } catch (java.security.NoSuchAlgorithmException nsa) {
           logger.log(Level.SEVERE, "NoSuchAlgorithmException", nsa);
           throw new MoaException(nsa);
        } catch (java.security.spec.InvalidKeySpecException ikse) {
            logger.log(Level.SEVERE, "InvalidKeySpecException", ikse);
            throw new MoaException(ikse);
        }
    }*/
}

