package moazs.ssl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

import moazs.util.MoaConfig;

import org.apache.axis.components.net.BooleanHolder;
import org.apache.axis.components.net.SecureSocketFactory;
import org.apache.log4j.Logger;

public class IAIKSecureSocketFactory implements SecureSocketFactory {

  private static Logger logger = Logger.getLogger(IAIKSecureSocketFactory.class);
  
  private static SSLSocketFactory socketFactory = null;
  
  public IAIKSecureSocketFactory(Hashtable properties) {
    if (properties != null) {
      Enumeration props = properties.keys();
      while (props.hasMoreElements()) {
        logger.info("IAIK SSL Trustmanager property: " + properties.get(props.nextElement()));
      }
    }
  }
  
  static {
    try {
      logger.info("Initializing IAIK Trustmanager.");
      MoaConfig mcfg = new MoaConfig();
      
      SSLContext context = SSLContext.getInstance("TLS");
      
      String keystore     = mcfg.getProperty("keystore");
      String keystorepw   = mcfg.getProperty("keystorepw") != null ? mcfg.getProperty("keystorepw") : "";
      String keystoretype = mcfg.getProperty("keystoretype") != null ? mcfg.getProperty("keystoretype") : "JKS"; 
  
      //String truststore   = mcfg.getProperty("truststore");
      //String truststorepw = mcfg.getProperty("truststorepw") != null ? mcfg.getProperty("truststorepw") : "";
      String truststoretype = mcfg.getProperty("truststoretype") != null ? mcfg.getProperty("truststoretype") : "JKS";

      // only initialize if we have a IAIK trustore
      if ("IAIK".equalsIgnoreCase(truststoretype)) {
        KeyManager[] keyManagers = null;

        KeyStore clientStore = KeyStore.getInstance(keystoretype);
        // just initialize keystore only if present
        if (keystore != null) {
          File keyStoreFile = new File(keystore);
          if (keyStoreFile.exists()) {
            FileInputStream fis = new FileInputStream(keystore);
            clientStore.load(fis, keystorepw.toCharArray());
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
            kmf.init(clientStore, keystorepw.toCharArray());
            keyManagers = kmf.getKeyManagers();
          }
        }
        context.init(keyManagers, new TrustManager[] { new IAIKTrustManager() }, new SecureRandom());
      }
      
      socketFactory = context.getSocketFactory();
      
    } catch(NoSuchAlgorithmException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    } catch (KeyStoreException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    } catch (UnrecoverableKeyException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    } catch (KeyManagementException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    } catch (CertificateException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    } catch (FileNotFoundException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    } catch (IOException e) {
      logger.warn("Fehler beim Initialisieren des IAIK Trustmanagers: ", e);
    }
  }
  
  public Socket create(String host, int port, StringBuffer stringbuffer,
      BooleanHolder booleanholder) throws IOException, UnknownHostException {
    if (port == -1) {
      port = 443;
    }
    return socketFactory.createSocket(host, port);
  }

}
