package moazs.ssl;

import iaik.logging.TransactionId;
import iaik.pki.DefaultPKIConfiguration;
import iaik.pki.DefaultPKIProfile;
import iaik.pki.PKIException;
import iaik.pki.PKIProfile;
import iaik.pki.SimplePKI;
import iaik.pki.pathvalidation.ChainingModes;
import iaik.pki.store.certstore.CertStoreParameters;
import iaik.pki.store.certstore.directory.DefaultDirectoryCertStoreParameters;
import iaik.pki.store.truststore.DefaultTrustStoreProfile;
import iaik.pki.store.truststore.TrustStoreTypes;
import iaik.utils.Util;

import java.io.File;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.X509TrustManager;

import org.apache.log4j.Logger;

import moazs.util.MoaConfig;

public class IAIKTrustManager implements X509TrustManager {

  private static Logger logger = Logger.getLogger(IAIKTrustManager.class);
  
  private static SimplePKI simplePKI = null;
  
  private static TransactionId tid = new TransactionIdImpl();
  
  static {
    try {
      
      MoaConfig mcfg = new MoaConfig();
      
      String truststoretype = mcfg.getProperty("truststoretype") != null ? mcfg.getProperty("truststoretype") : "JKS";

      if ("IAIK".equalsIgnoreCase(truststoretype)) {
        logger.info("Initializing IAIK PKI module.");
        // read root directory of IAIK truststore
        String sslBase = System.getProperty("javax.net.ssl.trustStore");
        
        simplePKI = new SimplePKI(tid);
        
        if (sslBase != null) {
          File sslBaseDir = new File(sslBase);
          if (!sslBaseDir.isDirectory()) {
            logger.warn("IAIK Truststore Base Verzeichnis '" + sslBaseDir.getAbsolutePath() + "' existiert nicht. Truststore konnte nicht initialisiert werden.");
          } else {
            String certStoreLocation = sslBase + "/certstore/";
            logger.debug("IAIK certstore location: " + certStoreLocation);
            String trustStoreLocation = sslBase + "/truststore/";
            logger.debug("IAIK truststore location: " + trustStoreLocation);
            
            logger.info("Initialisiere IAIK truststore.");
            DefaultDirectoryCertStoreParameters certParams = new DefaultDirectoryCertStoreParameters("moazs-cs-01",
                certStoreLocation,
                false,
                false);
            
            DefaultPKIConfiguration config = new DefaultPKIConfiguration( new CertStoreParameters[] { certParams });
            config.setChainingMode(ChainingModes.PKIX_MODE);
            
            DefaultTrustStoreProfile trustStoreProfile = new DefaultTrustStoreProfile("moazs-ts-01",
                TrustStoreTypes.DIRECTORY,
                trustStoreLocation);
                
            PKIProfile pkiProfile = new DefaultPKIProfile(trustStoreProfile);
      
            
            simplePKI.setConfiguration(config);
            simplePKI.setProfile(pkiProfile);
          }
        }
      }
    } catch(PKIException e) {
      logger.warn("Fehler beim Initialisieren des Trustmanagers: ", e);
    }
    
  }
  
  /* (non-Javadoc)
   * @see javax.net.ssl.X509TrustManager#checkClientTrusted(java.security.cert.X509Certificate[], java.lang.String)
   */
  public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
  }

  /* (non-Javadoc)
   * @see javax.net.ssl.X509TrustManager#checkServerTrusted(java.security.cert.X509Certificate[], java.lang.String)
   */
  public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
    try {
      if (!simplePKI.isCertificateValid(null, Util.convertCertificate(chain[0]))) {
        throw new CertificateException("SSL Zertifikat nicht gltig.");
      }
    } catch(PKIException e) {
      throw new CertificateException(e.getMessage());
    }
  }

  public X509Certificate[] getAcceptedIssuers() {
    return null;
  }

}
