package moazs.szr;

import java.io.File;
import java.net.URL;
import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import moazs.api.MoaException;
import moazs.util.MoaConfig;

import org.apache.axis.AxisFault;
import org.apache.log4j.Logger;

import at.gv.bmi.szr.FremdBPKRequestType;
import at.gv.bmi.szr.FremdBPKType;
import at.gv.bmi.szr.GetBPK;
import at.gv.bmi.szr.GetBPKResponse;
import at.gv.bmi.szr.GetStammzahl;
import at.gv.bmi.szr.GetStammzahlResponse;
import at.gv.bmi.szr.PersonInfoType;
import at.gv.bmi.szr.SZR;
import at.gv.bmi.szr.SZRServiceLocator;
import at.gv.bmi.szr.SZRSoapBindingStub;
import at.gv.bmi.szr.TransformBPK;
import at.gv.bmi.szr.person.PersonNameType;
import at.gv.bmi.szr.person.PhysicalPersonType;

public class SZRClient {

  private SZR stub;

  private SZRServiceLocator locator;

  private String keyStore = null;
  private String keyStorePassword = null;
  private String keyStoreType = null;
  
  private String trustStore = null;
  private String trustStorePassword;
  private String trustStoreType = null;
  
  private static Logger log_ = Logger.getLogger(SZRClient.class);
  
  private MoaConfig conf = null;
  
  //old client
  /*public SZRClient(URL url, String pvpFilePath, float pvpVersion, boolean isTestEnvironment)
      throws SZRClientException {

    
    File pvpFile = new File(pvpFilePath);

    try {
      log_.debug("Initializing SZR connection.");
      initConnection(url, pvpFile, pvpVersion, isTestEnvironment);
    } catch (ServiceException e) {
      throw new SZRClientException("Can't initialize the connection. "
          + e.getMessage());
    }

  }
  */
  
  // new client
  public SZRClient(URL url, boolean isTestEnvironment)
      throws SZRClientException {
    
    
    try {
      log_.debug("Initializing SZR connection.");
      conf = new MoaConfig();
      initConnection(url, null, -1, isTestEnvironment);
    } catch (ServiceException e) {
      throw new SZRClientException("Can't initialize the connection. "
          + e.getMessage());
    }
    
 }

  public void initConnection(URL url, File pvpFile, float pvpVersion, boolean isTestEnvironment)
      throws ServiceException {
    locator = new SZRServiceLocator();
    
    // new client
    if (isTestEnvironment) {
      stub = this.locator.getSZRTestumgebung(url);
    } else {
      stub = this.locator.getSZRProduktion(url);
    }
    // old client
    /*log_.debug("Getting szr connection.");
    stub = this.locator.getSZR(url);*/

    // pvp informationen
    // new client
    at.gv.bmi.szr.wsse.Security security = new at.gv.bmi.szr.wsse.Security();
    security.setPvpToken(conf.getPVPToken());
    ((SZRSoapBindingStub) stub).setHeader("http://schemas.xmlsoap.org/ws/2002/04/secext", "Security", security);
    
    
    // old client
    /*if (pvpFile != null) {
      if (pvpVersion ==  -1) {
        pvpVersion = Constants.PVP_STANDARD_VERSION;
      }
      log_.debug("Setting header.");
      try {
        PVPHeader pvpHeader = new PVPHeader(pvpVersion, pvpFile);
        ((SZRSoapBindingStub) this.stub).setPVPHeader(pvpHeader);
      } catch(Throwable t) {
        log_.warn("Fehler beim Initialisieren der PVP Verbindung.", t);
      }
      log_.debug("Setting header done.");
    }*/
  }

  /*public String processGetStammzahl(PersonInfoType personInfoType)
      throws SZRClientException, MandateException {
    try {
      log_.debug("Executing getStammzahl(personInfoType)");
      return stub.getStammzahl(personInfoType);
    } catch (RemoteException e) {
      if (e instanceof AxisFault) {
        AxisFault af = (AxisFault) e;
        // parse axisfault
        String codeString = af.getFaultCode().getLocalPart();
        throw new MandateException(codeString +" / " + af.getFaultString());
      } else
        throw new SZRClientException(e.getMessage());
    }
  }*/
  
  public String processGetOrganwalterBPK(PersonInfoType personInfoType) throws SZRClientException, MoaException {
    return processGetBPK(personInfoType, "urn:publicid:gv.at:cdid+OW");
  }
  

  public String processGetBPK(PersonInfoType personInfoType, String bereich)
      throws SZRClientException, MoaException {
    try {
      GetBPK getbpk = new GetBPK();
      getbpk.setPersonInfo(personInfoType);
      getbpk.setBereichsKennung(bereich);
      getbpk.setListMultiplePersons(null);
      getbpk.setVKZ("");
      GetBPKResponse response = stub.getBPK(getbpk);
      return response.getGetBPKReturn();
      
    } catch (RemoteException e) {
      if (e instanceof AxisFault) {
        AxisFault af = (AxisFault) e;
        // parse axisfault
        String codeString = af.getFaultCode().getLocalPart();
        throw new MoaException(codeString +" / " + af.getFaultString());
      } else
        throw new SZRClientException(e.getMessage());
    }
  }

  public FremdBPKType processTranslateBPK(PersonInfoType personInfoType,
      String bPK, String inputTarget, String outputTarget, String vkz, String reason)
      throws SZRClientException {

    FremdBPKRequestType fremdType = new FremdBPKRequestType();
    fremdType.setBereichsKennung(outputTarget);
    fremdType.setVKZ(vkz);
    FremdBPKRequestType[] target = new FremdBPKRequestType[] { fremdType };
    TransformBPK tbpk = new TransformBPK(personInfoType, bPK, inputTarget, reason, target); 
    try {
      return stub.transformBPK(tbpk)[0];
    } catch (RemoteException e) {
      if (e instanceof AxisFault) {
        AxisFault af = (AxisFault) e;
        // parse axisfault
        String codeString = af.getFaultCode().getLocalPart();
        int error = -1;
        try {
          error = Integer.parseInt(codeString.substring(1));
        } catch(NumberFormatException err) {
        }
        
        throw new SZRClientException(af.getFaultString(), error);
      } else {
        throw new SZRClientException(e.getMessage());
      }
    }
  }

  
  public String processGetStammzahl(PersonInfoType personInfoType)
      throws SZRClientException, MoaException {
    try {
      GetStammzahl getstammzahl = new GetStammzahl();
      getstammzahl.setPersonInfo(personInfoType);
      GetStammzahlResponse response = stub.getStammzahl(getstammzahl);
      return response.getStammzahl();
    } catch (RemoteException e) {
      if (e instanceof AxisFault) {
        AxisFault af = (AxisFault) e;
        // parse axisfault
        String codeString = af.getFaultCode().getLocalPart();
        throw new MoaException(codeString +" / " + af.getFaultString());
      } else
        throw new SZRClientException(e.getMessage());
    }
  }

  
  /*public IdentityLinkType processGetIdentityLink(PersonInfoType personInfoType,
      KeyValueType[] keys) throws SZRClientException {
    try {
      return stub.getIdentityLink(personInfoType, keys);
    } catch (RemoteException e) {
      if (e instanceof AxisFault) {
        e.printStackTrace();
        AxisFault af = (AxisFault) e;
        throw new SZRClientException(af.getFaultCode() + ": "
            + af.getFaultReason());
      } else
        throw new SZRClientException(e.getMessage());
    }
  }

  public FremdBPKType processTranslateBPK(PersonInfoType personInfoType,
      String bPK, String inputTarget, String outputTarget, String vkz, String reason)
      throws SZRClientException {

    FremdBPKRequestType fremdType = new FremdBPKRequestType();
    fremdType.setBereichsKennung(outputTarget);
    fremdType.setVKZ(vkz);
    FremdBPKRequestType[] target = new FremdBPKRequestType[] { fremdType };

    try {
      return stub.transformBPK(personInfoType, bPK, inputTarget,
          reason, target)[0];
    } catch (RemoteException e) {
      if (e instanceof AxisFault) {
        AxisFault af = (AxisFault) e;
        throw new SZRClientException(af.getFaultCode() + ": "
            + af.getFaultReason());
      } else
        throw new SZRClientException(e.getMessage());
    }
  }
  */
  public static PersonInfoType createPerson(String givenName, String familyName,
      String birthDay) throws SZRClientException {
      PersonNameType name = new PersonNameType();
      name.setGivenName(givenName);
      name.setFamilyName(familyName);
      PhysicalPersonType person = new PhysicalPersonType();
      person.setDateOfBirth(birthDay);
      person.setName(name);
      PersonInfoType personInfo = new PersonInfoType();
      personInfo.setPerson(person);
      return personInfo;
  }
  
  public static void main(String[] args) throws Exception {
    
  }

}
