/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package moazs.util;

import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import moazs.modell.Payload;
import moazs.modell.Postaladdress;
import moazs.modell.Request;
import moazs.modell.XmlDocument;

import org.apache.log4j.Logger;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
* Hilfsklasse die ein XmlDeckblatt fr nichtXML zustellungen erstellt
*/
public class MoaDeckblatt {

    private static String NS_MZS;
    private static String NS_P;
    private static String NS_XSI;
    static private Logger logger = Logger.getLogger(MoaDeckblatt.class);

    static {
        MoaConfig mcfg = new MoaConfig();

        NS_MZS = mcfg.getProperty("mzs", "Namespaces");
        NS_P   = mcfg.getProperty("p1",  "Namespaces");
        NS_XSI = mcfg.getProperty("xsi", "Namespaces");
    }


    /**
    * XmlDeckblatt erstellen falls kein XmlContent im Request mitgegeben wurde.
    */
    public XmlDocument makeDeckblatt(Request r) {
        XmlDocument db = new XmlDocument();
        db.setFilename("deckblatt.xml");
        db.setXmlProfileId("deckblatt");


        MoaConfig mcfg = new MoaConfig();
        Properties senderProb = mcfg.getSenderInfo(r.getSenderProfileId());

        try {
            DocumentBuilderFactory dobf = DocumentBuilderFactory.newInstance();
            DocumentBuilder dob = dobf.newDocumentBuilder();
            Document result = dob.newDocument();
            Element root = result.createElementNS(NS_MZS, "DeliveryInformation");
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns", NS_MZS);
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:p", NS_P);
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xsi", NS_XSI);

            Element sender = result.createElementNS(NS_MZS, "Sender");

            Element organisation = result.createElementNS(NS_MZS, "Organisation");
            Element fullName = result.createElementNS(NS_P, "p:FullName");
            fullName.appendChild(result.createTextNode(senderProb.getProperty("fullName")));

            organisation.appendChild(fullName);
            {
                Element postalAddress = result.createElementNS(NS_P, "p:PostalAddress");
                Element countryCode = result.createElementNS(NS_P, "p:CountryCode");
                countryCode.appendChild(result.createTextNode(senderProb.getProperty("countryCode")));
                Element postalCode = result.createElementNS(NS_P, "p:PostalCode");
                postalCode.appendChild(result.createTextNode(senderProb.getProperty("postalCode")));
                Element municipality = result.createElementNS(NS_P, "p:Municipality");
                municipality.appendChild(result.createTextNode(senderProb.getProperty("municipality")));

                Element deliveryAddress = result.createElementNS(NS_P, "p:DeliveryAddress");
                Element streetName = result.createElementNS(NS_P, "p:StreetName");
                streetName.appendChild(result.createTextNode(senderProb.getProperty("streetName")));
                Element buildingNumber = result.createElementNS(NS_P, "p:BuildingNumber");
                buildingNumber.appendChild(result.createTextNode(senderProb.getProperty("buildingNumber")));
                Element unit = result.createElementNS(NS_P, "p:Unit");
                unit.appendChild(result.createTextNode(senderProb.getProperty("unit")));

                deliveryAddress.appendChild(streetName);
                deliveryAddress.appendChild(buildingNumber);
                deliveryAddress.appendChild(unit);

                postalAddress.appendChild(countryCode);
                postalAddress.appendChild(postalCode);
                postalAddress.appendChild(municipality);
                postalAddress.appendChild(deliveryAddress);

                Element internetAddress = result.createElementNS(NS_P, "p:InternetAddress");
                Element addr = result.createElementNS(NS_P, "p:Address");
                internetAddress.appendChild(addr);
                addr.appendChild( result.createTextNode( senderProb.getProperty("senderEmail")));

                sender.appendChild(organisation);
                sender.appendChild(postalAddress);
                sender.appendChild(internetAddress);
            }

            root.appendChild(sender);

            Element receiver = result.createElementNS(NS_MZS, "Receiver");

            if (r.getRcpt().getPhysicalPerson()) {
                Element person = result.createElementNS(NS_P, "p:PhysicalPerson");
                Element name = result.createElementNS(NS_P, "p:Name");
                Element vname = result.createElementNS(NS_P, "p:GivenName");
                vname.appendChild(result.createTextNode(r.getRcpt().getGivenname()));
                Element nname = result.createElementNS(NS_P, "p:FamilyName");
                nname.appendChild(result.createTextNode(r.getRcpt().getFamilyname()));

                name.appendChild(vname);
                name.appendChild(nname);

                person.appendChild(name);

                //Date bd = r.getRcpt().getDateofbirth();
                //if (bd != null) {
                //    Element birthday = result.createElementNS(NS_P, "p:DateOfBirth");
                //    SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
                //    birthday.appendChild(result.createTextNode(fmt.format(r.getRcpt().getDateofbirth())));
                //    person.appendChild(birthday);
                //}

                receiver.appendChild(person);

            } else {
                Element company = result.createElementNS(NS_P, "p:CorporateBody");
                Element fname = result.createElementNS(NS_P, "p:FullName");
                fname.appendChild(result.createTextNode(r.getRcpt().getFullname()));

                company.appendChild(fname);
                receiver.appendChild(company);
            }

            Set pA = r.getRcpt().getPostaladdress();
            if (pA != null) {
                for (Iterator i = pA.iterator(); i.hasNext();) {
                    Postaladdress postal = (Postaladdress) i.next();
                    Element postalAddress = result.createElementNS(NS_P, "p:PostalAddress");
                    Element postalCode = result.createElementNS(NS_P, "p:PostalCode");
                    postalCode.appendChild(result.createTextNode(postal.getPostalcode()));
                    Element municipality = result.createElementNS(NS_P, "p:Municipality");
                    municipality.appendChild(result.createTextNode(postal.getMunicipality()));
                    Element deliveryAddress = result.createElementNS(NS_P, "p:DeliveryAddress");
                    Element streetName = result.createElementNS(NS_P, "p:StreetName");
                    streetName.appendChild(result.createTextNode(postal.getStreetname()));
                    Element buildingNr = result.createElementNS(NS_P, "p:BuildingNumber");
                    buildingNr.appendChild(result.createTextNode(postal.getBuilding()));

                    deliveryAddress.appendChild(streetName);
                    deliveryAddress.appendChild(buildingNr);

                    postalAddress.appendChild(postalCode);
                    postalAddress.appendChild(municipality);
                    postalAddress.appendChild(deliveryAddress);

                    receiver.appendChild(postalAddress);
                }
            }
            root.appendChild(receiver);

            Element meta = result.createElementNS(NS_MZS, "MetaInformation");
            Element mzsid = result.createElementNS(NS_MZS, "MZSDeliveryID");
            mzsid.appendChild( result.createTextNode( "MZS"+r.getId() ));

            Element deliveryquality = result.createElementNS(NS_MZS, "DeliveryQuality");
            deliveryquality.appendChild( result.createTextNode( r.getDeliveryQuality().getName()));

            Element date = result.createElementNS(NS_MZS, "Date");

            SimpleDateFormat fmtd = new SimpleDateFormat("yyyy-MM-dd");
            date.appendChild( result.createTextNode( fmtd.format(r.getTs())));  

            meta.appendChild(mzsid);
            meta.appendChild(deliveryquality);
            meta.appendChild(date);

            root.appendChild(date);

            Element attachments = result.createElementNS(NS_MZS, "Attachments");
            Set pl = r.getPayload();
            if (pl!=null) {
                for (Iterator i = pl.iterator(); i.hasNext(); ) {
                    Payload p = (Payload)i.next();
                    Element filename = result.createElementNS(NS_MZS, "FileName");
                    filename.appendChild( result.createTextNode( p.getFilename()));
                    attachments.appendChild(filename);
                }
            }
            root.appendChild( attachments );

            
            result.appendChild(root);
            XMLSerializer serializer = new XMLSerializer();
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            serializer.setOutputByteStream(bos);
    
            serializer.serialize(result);
            db.setData( bos.toByteArray());

        } catch (java.io.IOException ioe) {
            logger.warn("IO Exception", ioe);
        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            logger.warn("ParserConfigException", pce);
        }
        return db;
    }
}

