/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.util;

import java.util.List;

import moazs.async.MoaWorker;
import moazs.modell.QueueInfo;
import net.sf.workqueue.WorkQueue;
import net.sf.workqueue.WorkerFactory;
import net.sf.workqueue.WorkerThread;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.classic.Session;

public class MoaFactory {
    static private Logger logger = Logger.getLogger(MoaFactory.class);


    private static WorkQueue queue = null;
    private static SessionFactory sf = null;
    private static boolean canOrderSort = false;
    /**
     * Liefert die aktuelle Queue
     * @return WorkQueue
     */
    public static WorkQueue getQueue() {
        if (queue==null) {
            queue = new WorkQueue();
            MoaWorker goldmaster = new MoaWorker();
            WorkerThread.setMinLimit( 5 );
            WorkerFactory factory = WorkerFactory.getInstance();
            factory.setWorker(goldmaster);
            factory.setLimit(32);
            queue.setWorkerFactory(factory);
            factory.startWorker(queue);
        }
        return queue;
    }

    /**
     * Liefert die aktuelle Session, erzeugt im ersten Aufruf die Initialisierung des Mappings
     * @return Session
     */
    public static Session getSession() {
      return HibernateUtil.getCurrentSession();
    }
    /*public static Session getSession() {
        try {
            if (sf == null) {
                try {
                    MoaConfig mcfg = new MoaConfig();
                    if (mcfg.getProperty("hibernate.dialect", "Hibernate").equals("net.sf.hibernate.dialect.FirebirdDialect")) {
                        canOrderSort = true;
                    }
                    Configuration ds = new Configuration();
                    ds.addClass(QueueInfo.class);
                    ds.addClass(Status.class);
                    ds.addClass(Request.class);
                    ds.addClass(Payload.class);
                    ds.addClass(Rcpt.class);
                    ds.addClass(Identification.class);
                    ds.addClass(Internetaddress.class);
                    ds.addClass(Postaladdress.class);
                    ds.addClass(Telephone.class);
                    ds.addClass(DeliveryQuality.class);
                    ds.addClass(XmlDocument.class);
                    ds.addClass(Zuseserver.class);
                    ds.addProperties(mcfg.getHibernateProperties());
                    sf = ds.buildSessionFactory();
                } catch (MappingException me) {
                    logger.log(Level.SEVERE, "HibernateMappingException", me);
                } catch (HibernateException he) {
                    logger.log(Level.SEVERE, "HibernateException", he);
                }
                return sf.openSession();
            } else {
                return sf.openSession();
            }
        } catch (HibernateException he) {
            logger.log(Level.SEVERE, "HibernateException", he);
        }
        return null;
    }*/

    /**
     * @return int QueueID
     */
    public static int getQueueId()  {
        MoaConfig mcfg = new MoaConfig();
        return mcfg.getQueueId();
    }

    /**
     * @return Sting PingUrl
     */
    public static String getPingUrl()  {
        MoaConfig mcfg = new MoaConfig();
        return mcfg.getPingUrl();
    }


     /**
     * @return
     */
    public static int getMinload() {
        Session sess = HibernateUtil.getCurrentSession();
        try {
            List serverE = sess.find("select qi.id  from qi in class moazs.modell.QueueInfo where qi.id not in (select r.queueId from r in class moazs.modell.Request)");
            if (serverE.size() > 0) {
                return Integer.parseInt(((Long)serverE.get(0)).toString());
            }
            String query = "";
            if (canOrderSort) {     
                query = "select q.id as {qi.id}, q.url as {qi.url}, q.ts as {qi.ts}, count(r.id) as c from request r, queue_info q where r.queue_id = q.id group by q.id,q.url,q.ts order by count(r.id)";

            } else {
                query = "select q.id as {qi.id}, q.url as {qi.url}, q.ts as {qi.ts}, count(r.id) as c from request r, queue_info q where r.queue_id = q.id group by q.id,q.url,q.ts order by c";
            } 
            List server = sess.createSQLQuery(query, "qi", QueueInfo.class).list();
            //    return Integer.parseInt(((Object[])server.get(0))[0].toString());
            if (server.size()>0) {
                return (int)((QueueInfo)server.get(0)).getId();
            } else {
                return -1;
            } 
        } catch (HibernateException he) {
            logger.warn("HibernateException", he);
	    } catch (Exception e) {
            logger.warn("Exception", e);

        } finally {
            /*try {
                sess.connection().commit();
                sess.close();
            } catch (HibernateException he) {
                logger.log(Level.SEVERE, "HibernateException", he);
            } catch (java.sql.SQLException se) {
                logger.log(Level.SEVERE, "SQLException", se);
            }*/
        }
        return -1;
    }

}
