package moazs.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import moazs.api.MoaException;

import org.apache.log4j.Logger;
import org.apache.xerces.impl.Constants;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.parsers.IntegratedParserConfiguration;
import org.apache.xerces.parsers.XMLGrammarPreparser;
import org.apache.xerces.util.SymbolTable;
import org.apache.xerces.util.XMLGrammarPoolImpl;
import org.apache.xerces.xni.grammars.XMLGrammarDescription;
import org.apache.xerces.xni.grammars.XMLGrammarPool;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class MoaXMLUtils {

  private static XMLGrammarPool mzsGrammarPool = new XMLGrammarPoolImpl();
  private static XMLGrammarPool zuseGrammarPool = new XMLGrammarPoolImpl();
  
  /**
   * Name of the Xerces feature specifying explicite validation against a DTD.
   */
  protected static final String SAXFEATURE_VALIDATION_ = "http://xml.org/sax/features/validation";

  /**
   * Name of the Xerces feature specifying wheter or not to do namespace processing.
   */
  protected static final String SAXFEATURE_NAMESPACE_PROCESSING_ = "http://xml.org/sax/features/namespaces";

  /**
   * Name of the Xerces feature specifying explicite validation against an XML Schema, if such a Schema is
   * present.
   */
  protected static final String FEATURE_SCHEMA_VALIDATION_ =
    "http://apache.org/xml/features/validation/schema";

  /**
   * Name of the Xerces feature specifying dynamic validation depending on the presence of a grammar
   * specification.
   */
  protected static final String FEATURE_DYNAMIC_VALIDATION_ =
    "http://apache.org/xml/features/validation/dynamic";

  /**
   * Name of the Xerces feature specifying whether Xerces schema-normalizes element and attribute values in
   * case of validating parsing.
   */
  protected static final String FEATURE_SCHEMA_NORMALIZED_VALUES_ =
    "http://apache.org/xml/features/validation/schema/normalized-value";

  /**
   * Name of the Xerces feature specifying wheter or not continuing after a fatal error has occured.
   */
  protected static final String FEATURE_CONTINUE_AFTER_FATALERROR_ =
    "http://apache.org/xml/features/continue-after-fatal-error";

  /**
   * Name of the Xerces feature specifying wheter or not to include ignorable whitespace text nodes.
   */
  protected static final String FEATURE_INCLUDE_IGNORABLE_WHITESPACE_ =
    "http://apache.org/xml/features/dom/include-ignorable-whitespace";

  /**
   * Name of the Xerces feature specifying wheter or not to create entity reference nodes.
   */
  protected static final String FEATURE_CREATE_ENTITY_REF_NODES_ =
    "http://apache.org/xml/features/dom/create-entity-ref-nodes";

  /**
   * Name of the Xerces feature specifying wheter or not to use late node expansion.
   */
  protected static final String FEATURE_DEFER_NODE_EXPANSION_ =
    "http://apache.org/xml/features/dom/defer-node-expansion";

  /**
   * Name of the Xerces feature specifying wheter or not to augment Post-Schema-Validation-Infoset.
   */
  protected static final String FEATURE_INCLUDE_PSVI_NODES_ =
    "http://apache.org/xml/features/validation/schema/augment-psvi";

  /**
   * Name of the property telling the parser the location of the no-namespace schema to be used for
   * validation.
   */
  protected static final String PROPERTY_EXTERNAL_SCHEMALOCATION_ =
    "http://apache.org/xml/properties/schema/external-schemaLocation";

  /**
   * Name of the property telling the parser the location of the schemas to be used for validation.
   */
  protected static final String PROPERTY_EXTERNAL_NONAMESPACESCHEMALOCATION_ =
    "http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation"; 
  
  private static Logger logger = Logger.getLogger(MoaXMLUtils.class);
  
  // prepare grammar pools
  /*static {
    private void addSchemasToPool() {
      // adding schemas to pool
         logger.info("Adding MZS schema to pool.");
         try {
           DOMUtils.addSchemaToPool(Constants.class.getResourceAsStream(Constants.XMLDSIG_SCHEMA), Constants.DSIG_NS);
           DOMUtils.addSchemaToPool(Constants.class.getResourceAsStream(Constants.ECDSA_SCHEMA), Constants.ECDSA_NS);
           DOMUtils.addSchemaToPool(Constants.class.getResourceAsStream(Constants.SAML_SCHEMA), Constants.SAML_NS);
           DOMUtils.addSchemaToPool(Constants.class.getResourceAsStream(Constants.PERSONDATA_SCHEMA_MZS), Constants.P_NS);
           DOMUtils.addSchemaToPool(Constants.class.getResourceAsStream(Constants.MZS_SCHEMA), Constants.MZS_NS);
           DOMUtils.addSchemaToPool(Constants.class.getResourceAsStream(Constants.ZUS_SCHEMA), Constants.ZUSE_NS);
         } catch(IOException e) {
           logger.warn("Error while initializing schema pool.", e);
         }
       }
  }*/
  
  public static void initialize() {
    try {
      logger.info("Loading MZS schemas.");
      addSchemaToPool(mzsGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.XMLDSIG_SCHEMA), "UTF-8");
      addSchemaToPool(mzsGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.ECDSA_SCHEMA), "UTF-8");
      addSchemaToPool(mzsGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.SAML_SCHEMA), "UTF-8");
      addSchemaToPool(mzsGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.MZS_PERSONDATA_SCHEMA_MZS), "ISO-8859-1");
      addSchemaToPool(mzsGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.MZS_SCHEMA), "UTF-8");
      
      logger.info("Loading ZUSE schemas.");
      addSchemaToPool(zuseGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.XMLDSIG_SCHEMA), "UTF-8");
      addSchemaToPool(zuseGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.ECDSA_SCHEMA), "UTF-8");
      addSchemaToPool(zuseGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.SAML_SCHEMA), "UTF-8");
      addSchemaToPool(zuseGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.ZUSE_PERSONDATA_SCHEMA_MZS), "UTF-8");
      addSchemaToPool(zuseGrammarPool, MoaXMLUtils.class.getResourceAsStream(moazs.Constants.ZUS_SCHEMA), "UTF-8");
    } catch(IOException e) {
      logger.warn("Error adding schemas to schema pool.", e);
    }
  }
  
  private static void addSchemaToPool(XMLGrammarPool grammarPool, InputStream schema, String encoding) throws IOException {

    final String GRAMMAR_POOL = Constants.XERCES_PROPERTY_PREFIX + Constants.XMLGRAMMAR_POOL_PROPERTY;
    
    XMLGrammarPreparser grammarPreparser;
    SymbolTable sym = new SymbolTable(2039);
    grammarPreparser = new XMLGrammarPreparser(sym);
    grammarPreparser.registerPreparser(XMLGrammarDescription.XML_SCHEMA, null);
    grammarPreparser.setFeature(SAXFEATURE_NAMESPACE_PROCESSING_, true);
    grammarPreparser.setFeature("http://xml.org/sax/features/validation", true);
    grammarPreparser.setFeature("http://apache.org/xml/features/validation/schema", true);
    grammarPreparser.setFeature("http://apache.org/xml/features/validation/schema/augment-psvi", false);
    grammarPreparser.setFeature("http://apache.org/xml/features/validation/schema/normalized-value", false);
    grammarPreparser.setProperty(GRAMMAR_POOL, grammarPool); 
    XMLInputSource xmldsigSchemaXIS = new XMLInputSource(null, null, null, schema, encoding);
    grammarPreparser.preparseGrammar(XMLGrammarDescription.XML_SCHEMA, xmldsigSchemaXIS);
  }
  
  public static boolean validateMZS(Element element) throws MoaException {
    try {
      IntegratedParserConfiguration signatureParserConfig =
        new IntegratedParserConfiguration(new SymbolTable(2039), mzsGrammarPool);
      signatureParserConfig.setFeature(SAXFEATURE_NAMESPACE_PROCESSING_, true);
      signatureParserConfig.setFeature(SAXFEATURE_VALIDATION_, true);
      signatureParserConfig.setFeature(FEATURE_SCHEMA_VALIDATION_, true);
      signatureParserConfig.setFeature(FEATURE_INCLUDE_PSVI_NODES_, true); 
      signatureParserConfig.setFeature(FEATURE_SCHEMA_NORMALIZED_VALUES_, false);
      DOMParser domParser = new DOMParser(signatureParserConfig);
      domParser.setFeature(FEATURE_DEFER_NODE_EXPANSION_, true);
      domParser.setErrorHandler(new MoaXMLErrorHanlder());
      domParser.parse(new InputSource(new ByteArrayInputStream(serializeToByteArray(element, false))));
      return true;
    } catch (SAXException e) {
      throw new MoaException(e);
    } catch (IOException e) {
      throw new MoaException(e);
    }
  }
  
  public static boolean validateZUSE(Element element) throws MoaException {
    try {
      IntegratedParserConfiguration signatureParserConfig =
        new IntegratedParserConfiguration(new SymbolTable(2039), zuseGrammarPool);
      signatureParserConfig.setFeature(SAXFEATURE_NAMESPACE_PROCESSING_, true);
      signatureParserConfig.setFeature(SAXFEATURE_VALIDATION_, true);
      signatureParserConfig.setFeature(FEATURE_SCHEMA_VALIDATION_, true);
      signatureParserConfig.setFeature(FEATURE_INCLUDE_PSVI_NODES_, true); 
      signatureParserConfig.setFeature(FEATURE_SCHEMA_NORMALIZED_VALUES_, false);
      DOMParser domParser = new DOMParser(signatureParserConfig);
      domParser.setFeature(FEATURE_DEFER_NODE_EXPANSION_, true);
      domParser.setErrorHandler(new MoaXMLErrorHanlder());
      domParser.parse(new InputSource(new ByteArrayInputStream(serializeToByteArray(element, false))));
      return true;
    } catch (SAXException e) {
      throw new MoaException(e);
    } catch (IOException e) {
      throw new MoaException(e);
    }
  }

  
  public static void serialize(Document doc, OutputStream outputStream,
      boolean addXMLHeader, boolean ident) throws IOException {
      OutputFormat outFormat = null;
      XMLSerializer domSerializer;
      outFormat = new OutputFormat();
      outFormat.setEncoding("UTF-8");
      outFormat.setVersion("1.0");
      outFormat.setIndenting(ident);
      outFormat.setPreserveSpace(true);
      outFormat.setOmitXMLDeclaration(!addXMLHeader);
      domSerializer = new XMLSerializer(outputStream, outFormat);
      domSerializer.serialize(doc);
  }
  
  public static String serializeToString(Element doc, boolean addXMLHeader) throws IOException {
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    serialize(doc, bos, addXMLHeader, false);
    return bos.toString();
  }

  public static byte[] serializeToByteArray(Element doc, boolean addXMLHeader) throws IOException {
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    serialize(doc, bos, addXMLHeader, false);
    return bos.toByteArray();
  }
  
  public static byte[] serializeToByteArray(Document doc, boolean addXMLHeader) throws IOException {
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    serialize(doc, bos, addXMLHeader, false);
    return bos.toByteArray();
  }
  
  public static void serialize(Element element, OutputStream outputStream,
      boolean addXMLHeader, boolean ident) throws IOException {
      OutputFormat outFormat = null;
      XMLSerializer domSerializer;
      outFormat = new OutputFormat();
      outFormat.setEncoding("UTF-8");
      outFormat.setVersion("1.0");
      outFormat.setIndenting(ident);
      outFormat.setPreserveSpace(true);
      outFormat.setOmitXMLDeclaration(!addXMLHeader);
      domSerializer = new XMLSerializer(outputStream, outFormat);
      domSerializer.serialize(element);
  }

  public static Document createNewDocument() throws ParserConfigurationException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);
    return factory.newDocumentBuilder().newDocument();
  }
  
}

class MoaXMLErrorHanlder implements ErrorHandler {

  public void error(SAXParseException exception) throws SAXException {
    throw new SAXException(exception);
  }

  public void fatalError(SAXParseException exception) throws SAXException {
    throw new SAXException(exception);
  }

  public void warning(SAXParseException exception) throws SAXException {
    throw new SAXException(exception);
  }
  
}
