/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.util;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class MoaXPath {

    static private Logger logger = Logger.getLogger(MoaXPath.class);
    private static Element nsroot = null;
    static {
        try {
            MoaConfig mcfg = new MoaConfig();
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);

            DocumentBuilder db = dbf.newDocumentBuilder();

            Document nsholder = db.newDocument();
            nsroot = nsholder.createElementNS("http://reference.e-government.gv.at/namespace/moazs10/app2mzs#", "mzs:namespaceMapping");
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:mzs", mcfg.getProperty("mzs", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:p", mcfg.getProperty("p1", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:mss", mcfg.getProperty("mss", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:msp", mcfg.getProperty("msp", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:dsig", mcfg.getProperty("dsig", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:szr", mcfg.getProperty("szr", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xsi", mcfg.getProperty("xsi", "Namespaces"));
            nsroot.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:zus", mcfg.getProperty("zus", "Namespaces"));

            nsholder.appendChild(nsroot);
        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            logger.warn( "ParserConfigurationException", pce);
        }
    }

    /**
     * Liefert den ersten String
     * @param in Element
     * @param xpath XPath Expression
     * @return String
     */
    public static String getFirstString(Element in, String xpath) {
        if (in == null)
            return "";
        try {
            NodeList nl = XPathAPI.selectNodeList(in, xpath, nsroot);
            if (nl.getLength() > 0) {
                return (String) nl.item(0).getNodeValue();
            } else {
                return null;
            }
        } catch (javax.xml.transform.TransformerException te) {
            logger.warn( "TransformerException", te);
            return null;
        }
    }

    /**
     * Liefert erstest Element welches der Expression entspricht
     * @param in
     * @param xpath XPath Expression
     * @return Element
     */
    public static Element getFirstElement(Element in, String xpath) {
        if (in == null)
            return null;
        try {
            NodeList nl = XPathAPI.selectNodeList(in, xpath, nsroot);
            if (nl.getLength() > 0) {
                return (Element) nl.item(0);
            } else {
                return null;
            }
        } catch (javax.xml.transform.TransformerException te) {
            logger.warn( "TransformerException", te);
            return null;
        }
    }

    /**
     * Liefert ein Array aller Elemente
     * @param in Element
     * @param xpath XPath Expression
     * @return Elementarray
     */
    public static Element[] getAllElement(Element in, String xpath) {
        if (in == null)
            return null;
        try {
            NodeList nl = XPathAPI.selectNodeList(in, xpath, nsroot);
            Element[] result = new Element[nl.getLength()];
            for (int i = 0; i < nl.getLength(); i++) {
                result[i] = (Element) nl.item(i);
            }
            return result;
        } catch (javax.xml.transform.TransformerException te) {
            logger.warn( "TransformerException", te);
            return null;
        }
    }
}
