/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package moazs.watchdog;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import moazs.modell.QueueInfo;
import moazs.modell.Request;
import moazs.util.HibernateUtil;
import moazs.util.MoaConfig;
import moazs.util.MoaFactory;
import net.sf.workqueue.QueueException;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.log4j.Logger;
import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Transaction;
import org.hibernate.classic.Session;

public class Watchdog implements Runnable {
    private boolean stop;
    static private Logger logger = Logger.getLogger(Watchdog.class);

    private int watchdogSleep;
    private int watchdogLimit;

    /**
     * Konstruktor
     */
    public Watchdog() {
        stop = false;
    }
   
    public void end() {
        stop = true;
    }
   
    public void run() {
      
      Session s = HibernateUtil.getCurrentSession();
      Transaction tx = s.beginTransaction();
      try {
      
        MoaConfig mcfg = new MoaConfig();
        //mcfg.setLogging();
        logger.info("Configuring watchdog...");
        watchdogSleep = mcfg.getWatchdogSleep();
        watchdogLimit = mcfg.getWatchdogLimit();
        
        logger.info("Watchdog: start ...");

        Session sess = MoaFactory.getSession();
        List alone = sess.find("from request in class moazs.modell.Request where request.queueId = "+MoaFactory.getQueueId());
        if (alone.size()>0) {
            logger.debug("Watchdog: collecting Requests from breakdown ...");

        }
        
        for (Iterator i = alone.iterator(); i.hasNext(); ) {
            Request r = (Request)i.next();
            try {
                MoaFactory.getQueue().add( new Long( r.getId() ));
            } catch (QueueException qe) {
                r.setQueueId( -1 );
            }
        }
        tx.commit();
        //sess.flush();
      } catch(HibernateException e) {
        logger.warn( "Schwerer Fehler: " + e.getMessage());
        try {
          if (HibernateUtil.getCurrentSession().getTransaction().isActive()) {
            HibernateUtil.getCurrentSession().getTransaction().rollback();
          }
        } catch(HibernateException err) {
          logger.warn( "Could not rollback transaction.");
        }
      }
      
      // LOOP begins here
      while (!stop) {
          try {
              Thread.sleep( this.watchdogSleep );
          } catch (Exception e) {
            logger.warn( "Schwerer Fehler: " + e.getMessage());
          }
          try {
            tx = HibernateUtil.getCurrentSession().beginTransaction();
            renewTimestamp();
            checkDeadQueue();
            if (moazs.api.MoaSync.getCount()<5) { 
                enqueueWaiting();
            }
            tx.commit();
          } catch(HibernateException e) {
            logger.warn( "Schwerer Fehler: " + e.getMessage());
            try {
              if (HibernateUtil.getCurrentSession().getTransaction().isActive()) {
                HibernateUtil.getCurrentSession().getTransaction().rollback();
              }
            } catch(HibernateException err) {
              logger.warn( "Could not rollback transaction.");
            }
          }
          // checkState();
      }
      logger.info("Watchdog: stop ...");
    }

    /**
     * aktualisiert den TimeStamp
     */
    private void renewTimestamp() {
        Session sess = HibernateUtil.getCurrentSession();
        QueueInfo info = (QueueInfo)sess.get(QueueInfo.class, new Long(MoaFactory.getQueueId()));
        //sess.flush();
        if (info == null) {
            logger.debug("Watchdog: creating Timestamp ...");
            info = new QueueInfo();
            info.setId( MoaFactory.getQueueId());
            info.setTs( new Date());
            info.setUrl( MoaFactory.getPingUrl()  );
            sess.save(info);
        } else {
          info.setTs( new Date());  
        }
    }

    /**
     * Ueberprueft ob im Cluster andere Queues nicht mehr bearbeitet werden
     */
    private void checkDeadQueue() {
        Session sess = HibernateUtil.getCurrentSession();
          Date limit = new Date(new Date().getTime() - this.watchdogLimit );

          List dead = sess.find("from queue in class moazs.modell.QueueInfo where queue.ts < ?", limit, Hibernate.TIMESTAMP);

          for (Iterator i = dead.iterator(); i.hasNext(); ) {
              logger.info("Watchdog: found silent Queue ...");
              QueueInfo deadQueue = (QueueInfo)i.next();
              if ( !testAlive( deadQueue.getUrl() )) {
                  logger.info("Watchdog: Queue is dead ...");
                  List items = sess.find("from request in class moazs.modell.Request where queue_id = "+deadQueue.getId());
                  for (Iterator j = items.iterator(); j.hasNext(); ) {
                      Request r = (Request)j.next();
                      r.setQueueId(-1);
                  }
                  sess.delete(deadQueue);
              }
          }
    }

    /**
     * Setzt Requests mit QueueId=-1 auf die eigene QueueId
     */
    private void enqueueWaiting() {
        Session sess = HibernateUtil.getCurrentSession();
        Date limit = new Date();
        List alone = sess.find("from request in class moazs.modell.Request where request.queueId = -1 and request.retryTs < ?", limit, Hibernate.TIMESTAMP);
        for (Iterator i = alone.iterator(); i.hasNext(); ) {
            logger.info("Watchdog: lonly request ...");
            Request r = (Request)i.next();
            r.setQueueId( MoaFactory.getQueueId());
            try {
                MoaFactory.getQueue().add( new Long( r.getId() ));
            } catch (QueueException qe) {
                r.setQueueId( -1 );
            }
        }
        //sess.flush();
    }

    /**
     * Test alive
     * @param url
     * @return
     */
    private boolean testAlive(String url) {
        try {
            HttpClient client = new HttpClient();
            GetMethod gm = new GetMethod(url);
            client.executeMethod(gm);
            if (new String( gm.getResponseBody()).equals("pong")) {
                return true;
            }
        } catch (Exception e) {
            logger.warn( "Exception", e);
        }
        return false;
    }

    private void checkState() {
       logger.debug( "Queue : " +MoaFactory.getQueue().size());
       logger.debug( "Worker : " +MoaFactory.getQueue().getCount());
    }

}

