import java.io.IOException;
import java.util.Date;

import junit.framework.TestCase;
import moazs.async.MoaWorker;
import moazs.modell.DeliveryQuality;
import moazs.modell.Payload;
import moazs.modell.Rcpt;
import moazs.modell.Request;
import moazs.modell.Status;
import moazs.modell.XmlDocument;
import moazs.modell.Zuseserver;
import moazs.util.MoaFactory;
import net.sf.workqueue.WorkQueue;

import org.hibernate.HibernateException;
import org.hibernate.classic.Session;

public class TestEncryption extends TestCase {
    public TestEncryption(String name) {
        super(name);
    }

    public void testNoEncryption() {
        try {
            Session sess = MoaFactory.getSession();
            
            Request r = new Request();
            r.setStatus((Status) sess.load(Status.class, new Long(2)));
            r.setSenderProfileId("");
            r.setSignatureProfileId("");
            r.setDeliveryQuality((DeliveryQuality)sess.load(DeliveryQuality.class, new Long(0)));
            Rcpt rcpt = new Rcpt();
            sess.save(rcpt);
            r.setRcpt(rcpt);
            r.setTs(new Date());
            sess.save(r);
            Zuseserver z = new Zuseserver();
            sess.save(z);
            r.setZuseserver( z ) ;
            sess.flush(); 
            Payload p = new Payload();
            p.setData(new String("Foobar").getBytes());
            p.setRequest(r);
            r.addPayload(p);
            sess.save(p);
            sess.flush();
            sess.connection().commit();
            sess.close();

            MoaWorker mw = new MoaWorker( );
            mw.setQueue( new WorkQueue());
            mw.doSomething(new Long(r.getId()));

            sess = MoaFactory.getSession();
            r = (Request)sess.load(Request.class, new Long(r.getId()));
            Payload pn = (Payload)r.getPayload().toArray()[0];
            String result = new String(pn.getDataAsByteArray());
            assertTrue("MimeMessage", result.indexOf("Content-Type: multipart/mixed;")!=-1);
            assertTrue("Content", result.indexOf("Foobar")!=-1);
            
            sess.delete(r);
            sess.flush();
            sess.connection().commit();
            sess.close();
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("hibernate Exception");
        } catch (java.sql.SQLException se) {
            se.printStackTrace();
            fail("SQLException");
        } catch (IOException e) {
          e.printStackTrace();
          fail("hibernate Exception");
        }
    }

    public void testXmlMime() {
        try {
            Session sess = MoaFactory.getSession();
            
            Request r = new Request();
            r.setStatus((Status) sess.load(Status.class, new Long(2)));
            r.setSenderProfileId("");
            r.setSignatureProfileId("");
            r.setDeliveryQuality((DeliveryQuality)sess.load(DeliveryQuality.class, new Long(0)));
            Rcpt rcpt = new Rcpt();
            sess.save(rcpt);
            r.setRcpt(rcpt);
            Zuseserver z = new Zuseserver();
            sess.save(z);
            r.setZuseserver( z ) ;

            XmlDocument x = new XmlDocument();
            x.setFilename("foo.xml");
            x.setData( new String("<foo><bla/></foo>").getBytes());
            sess.save(x);
            r.setXmlDocument(x);
            r.setTs(new Date()); 
            sess.save(r);
            sess.flush();
            sess.connection().commit();
            sess.close();

            MoaWorker mw = new MoaWorker( );
            mw.setQueue( new WorkQueue());
            mw.doSomething(new Long(r.getId()));

            sess = MoaFactory.getSession();
            r = (Request)sess.load(Request.class, new Long(r.getId()));
            Payload pn = (Payload)r.getPayload().toArray()[0];
            String result = new String(pn.getDataAsByteArray());
            assertTrue("MimeMessage", result.indexOf("Content-Type: multipart/mixed;")!=-1);
            assertTrue("Content", result.indexOf("<foo><bla/></foo>")!=-1);
            
            sess.delete(r);
            sess.flush();
            sess.connection().commit();
            sess.close();
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("hibernate Exception");
        } catch (java.sql.SQLException se) {
            se.printStackTrace();
            fail("SQLException");
        } catch (IOException e) {
          e.printStackTrace();
          fail("SQLException");
        } 
    }

    public void testXmlMimePreview() {
        try {
            Session sess = MoaFactory.getSession();
            
            Request r = new Request();
            r.setStatus((Status) sess.load(Status.class, new Long(2)));
            r.setSenderProfileId("");
            r.setSignatureProfileId("");
            r.setDeliveryQuality((DeliveryQuality)sess.load(DeliveryQuality.class, new Long(0)));
            Rcpt rcpt = new Rcpt();
            sess.save(rcpt);
            r.setRcpt(rcpt);
            Zuseserver z = new Zuseserver();
            sess.save(z);
            r.setZuseserver( z ) ;

            XmlDocument x = new XmlDocument();
            x.setFilename("foo.xml");
            x.setData( new String("<foo><bla/></foo>").getBytes());
            x.setPstylesheetName("foo.xsl");
            x.setPstylesheet( new String("<blablabla/>").getBytes());

            sess.save(x);
            r.setXmlDocument(x);
            r.setTs(new Date()); 
            sess.save(r);
            sess.flush();
            sess.connection().commit();
            sess.close();

            MoaWorker mw = new MoaWorker( );
            mw.setQueue( new WorkQueue());
            mw.doSomething(new Long(r.getId()));

            sess = MoaFactory.getSession();
            r = (Request)sess.load(Request.class, new Long(r.getId()));
            Payload pn = (Payload)r.getPayload().toArray()[0];
            String result = new String(pn.getDataAsByteArray());
            assertTrue("MimeMessage", result.indexOf("Content-Type: multipart/mixed;")!=-1);
            assertTrue("Content", result.indexOf("<foo><bla/></foo>")!=-1);
            assertTrue("Styl", result.indexOf("<blablabla/>")!=-1);
           
            sess.delete(r);
            sess.flush();
            sess.connection().commit();
            sess.close();
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("hibernate Exception");
        } catch (java.sql.SQLException se) {
            se.printStackTrace();
            fail("SQLException");
        } catch (IOException e) {
          e.printStackTrace();
          fail("SQLException");
        } 
    }

    public void testEncryption() {
        try {
            Session sess = MoaFactory.getSession();
            
            Request r = new Request();
            r.setStatus((Status) sess.load(Status.class, new Long(2)));
            r.setSenderProfileId("");
            r.setSignatureProfileId("");
            r.setDeliveryQuality((DeliveryQuality)sess.load(DeliveryQuality.class, new Long(0)));
            Rcpt rcpt = new Rcpt();
            sess.save(rcpt);
            r.setRcpt(rcpt);
            r.setTs(new Date());
            sess.save(r);
            Zuseserver z = new Zuseserver();
            z.setCert( new String("MIIDNjCCAp+gAwIBAgIBADANBgkqhkiG9w0BAQQFADB2MQswCQYDVQQGEwJBVDEP\n"+
            "MA0GA1UECBMGdmllbm5hMQ8wDQYDVQQHEwZ2aWVubmExDDAKBgNVBAoTA2hwYzEM\n"+
            "MAoGA1UECxMDcm5kMQ0wCwYDVQQDEwRndXJ1MRowGAYJKoZIhvcNAQkBFgtndXJ1\n"+
            "QGhwYy5hdDAeFw0wNDA4MDUxNzQ4MTVaFw0wNDA5MDQxNzQ4MTVaMHYxCzAJBgNV\n"+
            "BAYTAkFUMQ8wDQYDVQQIEwZ2aWVubmExDzANBgNVBAcTBnZpZW5uYTEMMAoGA1UE\n"+
            "ChMDaHBjMQwwCgYDVQQLEwNybmQxDTALBgNVBAMTBGd1cnUxGjAYBgkqhkiG9w0B\n"+
            "CQEWC2d1cnVAaHBjLmF0MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQC4ouLh\n"+
            "Rtq9AMxRg2n9SQ/QlNTaerI6E1gn1X1qQcqRPWnTYeXT0sFCY8KcShE+GCpY8n3o\n"+
            "ukzRiVA4o2MFn21nZeWAqXAZysyRTYk4YzLDIuqvdNnt4KFdXZ75/4zRtiCnLa4u\n"+
            "AvQOdvBqJH/uNMadw0jYGhF26KDAayIcelDFJwIDAQABo4HTMIHQMB0GA1UdDgQW\n"+
            "BBR7wZ/VxFaGH0P2fJ/ABFRTrjHKdTCBoAYDVR0jBIGYMIGVgBR7wZ/VxFaGH0P2\n"+
            "fJ/ABFRTrjHKdaF6pHgwdjELMAkGA1UEBhMCQVQxDzANBgNVBAgTBnZpZW5uYTEP\n"+
            "MA0GA1UEBxMGdmllbm5hMQwwCgYDVQQKEwNocGMxDDAKBgNVBAsTA3JuZDENMAsG\n"+
            "A1UEAxMEZ3VydTEaMBgGCSqGSIb3DQEJARYLZ3VydUBocGMuYXSCAQAwDAYDVR0T\n"+
            "BAUwAwEB/zANBgkqhkiG9w0BAQQFAAOBgQCNUJEz4QCgEFDn4p0B/FGvUmR5Zp+G\n"+
            "+zxmaYbU3nSSabRT8oU7aWbr9uC4QP3zzjNn5W1LBNCvVVBsEni+VizZZLfeMaZ5\n"+
            "vvdMQN5tH7WXqlfS2Fv773MuN+EpL0cfEZvGAYmDIwMwG6GmWJlSUnX5Sw2J6UfV\n"+
            "fktV2bMfWQzyGA==\n").getBytes());; 
            sess.save(z);
            r.setZuseserver( z ) ;

            sess.flush(); 
            Payload p = new Payload();
            p.setData(new String("Foobar").getBytes());
            p.setRequest(r);
            r.addPayload(p);
            sess.save(p);
            sess.flush();
            sess.connection().commit();
            sess.close();

            MoaWorker mw = new MoaWorker( );
            mw.setQueue( new WorkQueue());
            mw.doSomething(new Long(r.getId()));

            sess = MoaFactory.getSession();
            r = (Request)sess.load(Request.class, new Long(r.getId()));
            Payload pn = (Payload)r.getPayload().toArray()[0];
            
//            System.out.println("------------------------------------------------");
//            System.out.println(new String(pn.getData()));
//            System.out.println("------------------------------------------------");
           
            assertTrue("encrypted ", new String(pn.getDataAsByteArray()).indexOf("MIAGCSqGSIb3DQEHA6CAMIACAQAxggEWMIIBEgIBADB7MHYxCzAJBgNVBAYTAkFUMQ8wDQYDVQQI")!=-1);
            sess.delete(r);
            sess.flush();
            sess.connection().commit();
            sess.close();
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("hibernate Exception");
        } catch (java.sql.SQLException se) {
            se.printStackTrace();
            fail("SQLException");
        } catch (IOException e) {
          e.printStackTrace();
          fail("SQLException");
        }
    }
}
