import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;
import moazs.api.MoaMapper;
import moazs.modell.Payload;
import moazs.modell.Request;
import moazs.modell.XmlDocument;
import moazs.util.MoaFactory;

import org.hibernate.HibernateException;
import org.hibernate.classic.Session;
import org.w3c.dom.Document;
public class TestPayload extends TestCase {
    public TestPayload(String name) {
        super(name);
    }

    public void testXMLcontent() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <XMLDocument>\n"+
                "     <XMLContent>\n"+
                "       <foobar/>\n"+
                "     </XMLContent>\n"+
                "     <FileName>text.xml</FileName>\n"+
                "     <MIMEType>text/xml</MIMEType>\n"+
                "     <ResultingMIMEType>text/html</ResultingMIMEType>\n"+
                "     <SignatureXPath Index=\"42\">/foobar</SignatureXPath>\n"+
                "     <PreviewStylesheet>\n"+
                "       <FileName>foo.xsl</FileName>\n"+
                "       <XMLContent><xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1990/XSL/Transform\" version=\"1.0\"/></XMLContent>\n"+
                "     </PreviewStylesheet>\n"+
                "     <SignatureStylesheet>\""+
                "       <XMLContent><xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1990/XSL/Transform\" version=\"1.0\"/></XMLContent>\n"+
                "     </SignatureStylesheet>\n"+
                "   </XMLDocument>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
            Request r = new Request();
            Session sess = MoaFactory.getSession();
            MoaMapper.getPayload(sess, r, in);
            XmlDocument doc = r.getXmlDocument();
            assertEquals("Filename", "text.xml", doc.getFilename());
            assertEquals("MIMEType", "text/xml", doc.getMimetype());
            assertEquals("Resulting MIMEType", "text/html", doc.getResultingMimetype());
            assertEquals("XPath", "/foobar", doc.getXpath());
            assertEquals("XPathIndex", 42, doc.getXpathIndex());

            assertEquals("PreviewStylesheetName", "foo.xsl", doc.getPstylesheetName());
            assertEquals("SStylesheet", "<?xml version=\"1.0\"?>\n<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1990/XSL/Transform\"/>", new String(doc.getSStyleSheetAsByteArray()));
            assertEquals("PStylesheet", "<?xml version=\"1.0\"?>\n<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1990/XSL/Transform\"/>", new String(doc.getPStyleSheetAsByteArray()));
            assertEquals("Data", "<?xml version=\"1.0\"?>\n<foobar/>", new String(doc.getDataAsByteArray()));
            assertEquals("PreviewStylesheetName", "foo.xsl", doc.getPstylesheetName());

            sess.connection().rollback();
            sess.close();

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("HibernateException");
        } catch (moazs.api.MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        } catch (java.sql.SQLException sqle) {
            sqle.printStackTrace();
            fail("SQLException");
        }

    }

    public void testXMLprofile() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <XMLDocument>\n"+
                "     <XMLContent>\n"+
                "       <foobar/>\n"+
                "     </XMLContent>\n"+
                "     <XMLProfileID>xmlprofile1</XMLProfileID>\n"+
                "   </XMLDocument>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
            Request r = new Request();
            Session sess = MoaFactory.getSession();
            MoaMapper.getPayload(sess, r, in);
            XmlDocument doc = r.getXmlDocument();
            assertEquals("XMLProfileID", "xmlprofile1", doc.getXmlProfileId());
            assertEquals("Data", "<?xml version=\"1.0\"?>\n<foobar/>", new String(doc.getDataAsByteArray()));

            sess.connection().rollback();
            sess.close();

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("HibernateException");
        } catch (moazs.api.MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        } catch (java.sql.SQLException sqle) {
            sqle.printStackTrace();
            fail("SQLException");
        }

    }

    public void testBase64() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <Payload>\n"+
                "     <BinaryDocument>\n"+
                "       <Base64Content>AAAAAA==</Base64Content>\n"+
                "       <FileName>rechtsmittel.pdf</FileName>\n"+
                "       <MIMEType>application/x-pdf</MIMEType>\n"+
                "     </BinaryDocument>\n"+
                "   </Payload>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
            Request r = new Request();
            Session sess = MoaFactory.getSession();
            MoaMapper.getPayload(sess, r, in);

            Payload p = (Payload)r.getPayload().toArray()[0];
            assertEquals("Mimetype", "application/x-pdf", p.getMimetype());
            assertEquals("Filename", "rechtsmittel.pdf", p.getFilename());
            assertEquals("data", "AAAAAA==", new String(p.getDataAsByteArray()));

            sess.connection().rollback();
            sess.close();

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("HibernateException");
        } catch (moazs.api.MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        } catch (java.sql.SQLException sqle) {
            sqle.printStackTrace();
            fail("SQLException");
        }

    }

    public void testCallback() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <Payload>\n"+
                "     <DocumentReference>\n"+
                "       <URL>http://blabla.com/document.doc</URL>\n"+
                "       <MD5Checksum>42</MD5Checksum>\n"+
                "       <MIMEType>application/blarg</MIMEType>\n"+
                "       <FileName>document.doc</FileName>\n"+
                "     </DocumentReference>\n"+
                "   </Payload>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
            Request r = new Request();
            Session sess = MoaFactory.getSession();
            MoaMapper.getPayload(sess, r, in);

            Payload p = (Payload)r.getPayload().toArray()[0];
            assertEquals("url", "http://blabla.com/document.doc", p.getUrl());
            assertEquals("Checksum", "42", p.getMd5checksum());
            assertEquals("Mimetype", "application/blarg", p.getMimetype());
            assertEquals("Filename", "document.doc", p.getFilename());

            sess.connection().rollback();
            sess.close();

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("HibernateException");
        } catch (moazs.api.MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        } catch (java.sql.SQLException sqle) {
            sqle.printStackTrace();
            fail("SQLException");
        }

    }

}
