import java.io.ByteArrayInputStream;
import java.text.SimpleDateFormat;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;
import moazs.api.MoaMapper;
import moazs.modell.Identification;
import moazs.modell.Internetaddress;
import moazs.modell.Postaladdress;
import moazs.modell.Rcpt;
import moazs.modell.Telephone;
import moazs.util.MoaFactory;

import org.hibernate.HibernateException;
import org.hibernate.classic.Session;
import org.w3c.dom.Document;
public class TestRcpt extends TestCase {
    public TestRcpt(String name) {
        super(name);
    }

    public void testRcptPhysical() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <Receiver>\n" +
                "    <p:PhysicalPerson>\n" +
                "      <p:Name>\n" +
                "        <p:GivenName>Hermann</p:GivenName>\n" +
                "        <p:FamilyName>Maier</p:FamilyName>\n" +
                "      </p:Name>\n" +
                "      <p:DateOfBirth>1968-11-02</p:DateOfBirth>\n" +
                "    </p:PhysicalPerson>\n" +
                "  </Receiver>\n" +
                "</DeliveryRequest>\n"
            ).getBytes()));
       
            try {
                Session sess = MoaFactory.getSession();
                Rcpt result = MoaMapper.saveRcpt(sess, in);
                assertEquals("Givenname", "Hermann", result.getGivenname()); 
                assertEquals("Familyname", "Maier", result.getFamilyname());
                String date = "";
                date = new SimpleDateFormat("yyyy-MM-dd").format(result.getDateofbirth());
                assertEquals("Dateofbirth", "1968-11-02", date);

                sess.connection().rollback();
                sess.close();
            } catch (HibernateException he) {
                he.printStackTrace();
                fail("Hibernate exception");
            } catch (java.sql.SQLException se) {
                se.printStackTrace();
                fail("Sql Exception");
            } catch (moazs.api.MoaException me) {
                me.printStackTrace();
                fail("MoaException");
            }


        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

    public void testRcptPhysicalOptional() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            Document in = builder.parse(new ByteArrayInputStream(new String(
                            "<?xml version=\"1.0\"?>\n" +
                            "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                            "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                            "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                            "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                            "  <Receiver>\n" +
                            "    <p:PhysicalPerson>\n" +
                            "      <p:Name>\n" +
                            "        <p:GivenName>Hermann</p:GivenName>\n" +
                            "        <p:FamilyName p:prefix=\"foo\" p:primary=\"false\">Maier</p:FamilyName>\n" +
                            "      </p:Name>\n" +
                            "      <p:DateOfBirth>1968-11-02</p:DateOfBirth>\n" +
                            "    </p:PhysicalPerson>\n" +
                            "  </Receiver>\n" +
                            "</DeliveryRequest>\n"
                            ).getBytes()));

            try {
                Session sess = MoaFactory.getSession();
                Rcpt result = MoaMapper.saveRcpt(sess, in);
                assertEquals("Givenname", "Hermann", result.getGivenname()); 
                assertEquals("Familyname", "Maier", result.getFamilyname());
                String date = "";
                date = new SimpleDateFormat("yyyy-MM-dd").format(result.getDateofbirth());
                assertEquals("Dateofbirth", "1968-11-02", date);
                assertEquals("Prefix", "foo", result.getPrefix());
                assertEquals("Primary", false, result.getPrimary());

                sess.connection().rollback();
                sess.close();
            } catch (HibernateException he) {
                he.printStackTrace();
                fail("Hibernate exception");
            } catch (java.sql.SQLException se) {
                se.printStackTrace();
                fail("Sql Exception");
            } catch (moazs.api.MoaException me) {
                me.printStackTrace();
                fail("MoaException");
            }


        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

    public void testRcptCorporate() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            Document in = builder.parse(new ByteArrayInputStream(new String(
                            "<?xml version=\"1.0\"?>\n" +
                            "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                            "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                            "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                            "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                            "  <Receiver>\n" +
                            "    <p:CorporateBody>\n" +
                            "        <p:FullName>Abteilung Objektschutz</p:FullName>\n" +
                            "        <p:Organisation>Mafia</p:Organisation>\n" +
                            "    </p:CorporateBody>\n" +
                            "  </Receiver>\n" +
                            "</DeliveryRequest>\n"
                            ).getBytes()));

            try {
                Session sess = MoaFactory.getSession();
                Rcpt result = MoaMapper.saveRcpt(sess, in);
                assertEquals("Organisation", "Mafia", result.getOrganisation());
                assertEquals("Fullname", "Abteilung Objektschutz", result.getFullname()); 
                sess.connection().rollback();
                sess.close();
            } catch (HibernateException he) {
                he.printStackTrace();
                fail("Hibernate exception");
            } catch (java.sql.SQLException se) {
                se.printStackTrace();
                fail("Sql Exception");
            } catch (moazs.api.MoaException me) {
                me.printStackTrace();
                fail("MoaException");
            }


        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

    public void testRcptAddress() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            Document in = builder.parse(new ByteArrayInputStream(new String(
                            "<?xml version=\"1.0\"?>\n" +
                            "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                            "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                            "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                            "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                            "  <Receiver>\n" +
                            "    <p:PhysicalPerson>\n" +
                            "      <p:Name>\n" +
                            "        <p:GivenName>Hermann</p:GivenName>\n" +
                            "        <p:FamilyName>Maier</p:FamilyName>\n" +
                            "      </p:Name>\n" +
                            "      <p:DateOfBirth>1968-11-02</p:DateOfBirth>\n" +
                            "    </p:PhysicalPerson>\n" +
                            "    <p:PostalAddress>\n" +
                            "       <p:PostalCode>1010</p:PostalCode>\n" +
                            "       <p:Municipality>Wien</p:Municipality>\n" +
                            "       <p:DeliveryAddress>\n" +
                            "         <p:StreetName>Am Graben</p:StreetName>\n" +
                            "         <p:BuildingNumber>1</p:BuildingNumber>\n" +
                            "       </p:DeliveryAddress>\n" +
                            "     </p:PostalAddress>\n" +
                            "     <p:InternetAddress>\n" +
                            "       <p:Address>hermann.maier@superg.com</p:Address>\n" +
                            "     </p:InternetAddress>\n" +
                            "     <p:TelephoneAddress>\n" +
                            "       <p:Number>+ 43 1 2345</p:Number>\n" +
                            "     </p:TelephoneAddress>\n" +
                            "  </Receiver>\n" +
                            "</DeliveryRequest>\n"
                            ).getBytes()));

            try {
                Session sess = MoaFactory.getSession();
                Rcpt result = MoaMapper.saveRcpt(sess, in);

                Set tel = result.getTelephone();
                assertNotNull("Telephone gefunden", tel);
                assertEquals("Telephone anzahl", 1, tel.size());

                Telephone t = (Telephone)tel.toArray()[0];
                assertEquals("Nummer", "+ 43 1 2345", t.getValue());


                Set email = result.getInternetaddress();
                assertNotNull("Internet gefunden", email);
                assertEquals("Internet anzahl", 1, email.size());

                Internetaddress i = (Internetaddress)email.toArray()[0];
                assertEquals("email", "hermann.maier@superg.com", i.getValue());

                Set addr = result.getPostaladdress();
                assertNotNull("Addresse gefunden", addr);
                assertEquals("Addresse anzahl", 1, addr.size());

                Postaladdress a = (Postaladdress)addr.toArray()[0];
                assertEquals("postalcode", "1010", a.getPostalcode());
                assertEquals("municipality", "Wien", a.getMunicipality());
                assertEquals("streetname", "Am Graben", a.getStreetname());
                assertEquals("buildingNumber", "1", a.getBuilding());

                sess.connection().rollback();
                sess.close();
            } catch (HibernateException he) {
                he.printStackTrace();
                fail("Hibernate exception");
            } catch (java.sql.SQLException se) {
                se.printStackTrace();
                fail("Sql Exception");
            } catch (moazs.api.MoaException me) {
                me.printStackTrace();
                fail("MoaException");
            }


        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

    public void testRcptId() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            Document in = builder.parse(new ByteArrayInputStream(new String(
                            "<?xml version=\"1.0\"?>\n" +
                            "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                            "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                            "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                            "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                            "  <Receiver>\n" +
                            "    <p:PhysicalPerson>\n" +
                            "      <p:Name>\n" +
                            "        <p:GivenName>Hermann</p:GivenName>\n" +
                            "        <p:FamilyName>Maier</p:FamilyName>\n" +
                            "      </p:Name>\n" +
                            "      <p:DateOfBirth>1968-11-02</p:DateOfBirth>\n" +
                            "    </p:PhysicalPerson>\n" +
                            "    <p:Identification>\n" +
                            "       <p:Type>fooid</p:Type>\n" +
                            "       <p:Value>1234</p:Value>\n" +
                            "    </p:Identification>\n" +
                            "  </Receiver>\n" +
                            "</DeliveryRequest>\n"
                            ).getBytes()));

            try {
                Session sess = MoaFactory.getSession();
                Rcpt result = MoaMapper.saveRcpt(sess, in);

                Set id = result.getIdentification();
                assertNotNull("Identification gefunden", id);
                assertEquals("Identification anzahl", 1, id.size());

                Identification i = (Identification)id.toArray()[0];
                assertEquals("type", "fooid", i.getIdType());
                assertEquals("value", "1234", i.getValue());

                sess.connection().rollback();
                sess.close();
            } catch (HibernateException he) {
                he.printStackTrace();
                fail("Hibernate exception");
            } catch (java.sql.SQLException se) {
                se.printStackTrace();
                fail("Sql Exception");
            } catch (moazs.api.MoaException me) {
                me.printStackTrace();
                fail("MoaException");
            }

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

}
