import java.io.IOException;
import java.util.Date;
import java.util.HashSet;

import junit.framework.TestCase;
import moazs.async.MoaWorker;
import moazs.external.MoaSS;
import moazs.modell.DeliveryQuality;
import moazs.modell.Payload;
import moazs.modell.Rcpt;
import moazs.modell.Request;
import moazs.modell.Status;
import moazs.modell.XmlDocument;
import moazs.modell.Zuseserver;
import moazs.util.MoaFactory;
import net.sf.workqueue.WorkQueue;

import org.hibernate.HibernateException;
import org.hibernate.classic.Session;

public class TestSS extends TestCase {
    public TestSS(String name) {
        super(name);
    }

    public void testSign() {
        Session sess = MoaFactory.getSession();
        HashSet payloads = new HashSet();
        Payload p1 = new Payload();
        p1.setData( new String("Foobar").getBytes());
        p1.setFilename("foobar.txt");
        p1.setMimetype("text/plain");
        Payload p2 = new Payload();
        p2.setData( new String("blabla bla blabla").getBytes());
        p2.setFilename("foobar.xml");
        p2.setMimetype("application/blabla");
        payloads.add(p1);
        payloads.add(p2);
        MoaSS mss = new MoaSS();

        XmlDocument deckblatt = new XmlDocument();
        deckblatt.setData( new String("<?xml version=\"1.0\"?>\n<foobar/>").getBytes());
        deckblatt.setSstylesheet( new String("<?xml version=\"1.0\"?>\n<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"/>").getBytes());
        deckblatt.setXpath("/foobar");
        deckblatt.setXpathIndex(0);
        String result = "";
        try {
            result = new String(mss.sign(deckblatt, payloads, "Signaturdienst_I", sess));
        } catch ( moazs.api.MoaException me) {
            fail("MoaException");
        }
        assertTrue("Signature ist da wo sie sein soll", result.indexOf("<?xml version=\"1.0\"?>\n<foobar xmlns=\"\"><dsig:Signature")==0);

    }


    public void testNoEncryption() {
        try {
            Session sess = MoaFactory.getSession();

            Request r = new Request();
            r.setStatus((Status) sess.load(Status.class, new Long(1)));
            r.setSenderProfileId("");
            r.setSignatureProfileId("Signaturdienst_I");
            r.setDeliveryQuality((DeliveryQuality)sess.load(DeliveryQuality.class, new Long(0)));
            Rcpt rcpt = new Rcpt();
            sess.save(rcpt);
            r.setRcpt(rcpt);
            r.setTs(new Date());
            sess.save(r);
            Zuseserver z = new Zuseserver();
            sess.save(z);
            r.setZuseserver( z ) ;
            sess.flush();
            Payload p1 = new Payload();
            p1.setData( new String("Foobar").getBytes());
            p1.setFilename("foobar.txt");
            p1.setMimetype("text/plain");
            p1.setRequest(r);
            sess.save(p1);
            Payload p2 = new Payload();
            p2.setData( new String("blabla bla blabla").getBytes());
            p2.setFilename("foobar.xml");
            p2.setMimetype("application/blabla");
            p2.setRequest(r);
            sess.save(p2);
            r.addPayload(p1);
            r.addPayload(p2);

            XmlDocument deckblatt = new XmlDocument();
            deckblatt.setData( new String("<?xml version=\"1.0\"?>\n<foobar/>").getBytes());
            deckblatt.setSstylesheet( new String("<?xml version=\"1.0\"?>\n<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"/>").getBytes());
            deckblatt.setXpath("/foobar");
            deckblatt.setXpathIndex(0);
            sess.save(deckblatt);
            r.setXmlDocument(deckblatt);
             sess.flush();
            sess.connection().commit();
            sess.close();

            MoaWorker mw = new MoaWorker( );
            mw.setQueue( new WorkQueue());
            mw.doSomething(new Long(r.getId()));

            sess = MoaFactory.getSession();
            r = (Request)sess.load(Request.class, new Long(r.getId()));
            XmlDocument rx = r.getXmlDocument();
            String result = new String(rx.getDataAsByteArray());
            //System.out.println( result );
            assertTrue("Signature ist da wo sie sein soll", result.indexOf("<?xml version=\"1.0\"?>\n<foobar xmlns=\"\"><dsig:Signature")==0);

            sess.delete(r);
            sess.flush();
            sess.connection().commit();
            sess.close();
        } catch (HibernateException he) {
            he.printStackTrace();
            fail("hibernate Exception");
        } catch (java.sql.SQLException se) {
            se.printStackTrace();
            fail("SQLException");
        } catch (IOException se) {
          se.printStackTrace();
          fail("IOException");
      }
    }

}
