import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;
import moazs.api.MoaException;
import moazs.api.MoaMapper;
import moazs.modell.Request;
import moazs.util.MoaFactory;

import org.hibernate.HibernateException;
import org.hibernate.classic.Session;
import org.w3c.dom.Document;
public class TestSender extends TestCase {
    public TestSender(String name) {
        super(name);
    }

    public void testSender() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <Sender>\n"+
                "    <ProfileID>123456</ProfileID>\n"+
                "    <SignatureKeyID>1111</SignatureKeyID>\n"+
                "  </Sender>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
        
            //assertEquals("Sender id", "123456",  MoaMapper.getSenderProfileId(in)); 
            assertEquals("Signature id", "1111", MoaMapper.getSignatureProfileId(in));

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }
    
    public void testSenderMissing() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <Sender>\n"+
                "  </Sender>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
        
            Session sess = MoaFactory.getSession();
            try {
                Request r = MoaMapper.saveRequest(sess, in);
                sess.connection().rollback();
            } catch (moazs.api.MoaException me) {
                return ;
            } catch (HibernateException he) {
                fail("Hibernate Exeption");
            } catch (java.sql.SQLException se) {
                fail("SQL Exeption");
            } finally {
                try {
                    sess.close();
                } catch (HibernateException he) {
                    fail("Hibernate Exception");
                }
            }
            fail("Exception missing");

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

    public void testMetadata() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <MetaData>\n"+
                "    <AppDeliveryID>4711</AppDeliveryID>\n"+
                "    <DeliveryQuality>RSa</DeliveryQuality>\n"+
                "    <RequiresEncryption>true</RequiresEncryption>\n"+
                "  </MetaData>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
        
            assertEquals("app id", "4711", MoaMapper.getAppDeliveryId(in)); 
            assertEquals("delivery Quality", "RSa", MoaMapper.getDeliveryQuality(in));
            assertEquals("Requires Encryption", true, MoaMapper.getRequiresEncryption(in));            

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        } catch (moazs.api.MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        }
    }

    public void testMetadataMissing() {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            
            Document in = builder.parse(new ByteArrayInputStream(new String(
                "<?xml version=\"1.0\"?>\n" +
                "<DeliveryRequest xmlns=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs#\"\n"+
                "                xmlns:p=\"http://reference.e-government.gv.at/namespace/persondata/20020228#\"\n"+
                "                xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
                "                xsi:schemaLocation=\"http://reference.e-government.gv.at/namespace/moazs10/app2mzs# ../schema/mzs/app2mzs.xsd\">\n"+
                "  <MetaData>\n"+
                "  </MetaData>\n"+
                "</DeliveryRequest>\n"
            ).getBytes()));
            Session sess = MoaFactory.getSession();
            try {
                Request r = MoaMapper.saveRequest(sess, in);
                sess.connection().rollback();
            } catch (moazs.api.MoaException me) {
                return ;
            } catch (HibernateException he) {
                fail("Hibernate Exeption");
            } catch (java.sql.SQLException se) {
                fail("SQL Exeption");
            } finally {
                try {
                    sess.close();
                } catch (HibernateException he) {
                    fail("Hibernate Exception");
                }                    
            }
            fail("Exception missing");

        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            pce.printStackTrace();
            fail("Parser Configuration exception");
        } catch (org.xml.sax.SAXException se) {
            se.printStackTrace();
            fail("Sax Exception");
        } catch (java.io.IOException ioe) {
            ioe.printStackTrace();
            fail("IOException");
        }
    }

}
