/*
 * Created on 10-Aug-2004
 *
 * by Rainer Treutlein
 *
 * Project: moazs  2004
 * Release:
 *
 */

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import moazs.api.MoaException;
import moazs.external.ZkopfRequest;
import moazs.modell.Zuseserver;

/**
 * JUnit Tests fuer den ZuseRequest
 * 
 * @author Rainer Treutlein on 10-Aug-2004
 * 
 *  
 */
public class TestZuseRequest extends TestCase {

    private int nrOfTests = 5;
    private ZkopfRequest zre = null;

    private static List inputMimeTypes = null;
    
    static {
      inputMimeTypes = new ArrayList();
      inputMimeTypes.add("text/xml");
    }
    
    public void testGetZuseInfonoError() {

        for (int i = 1; i < nrOfTests + 1; i++) {
            zre = new ZkopfRequest(inputMimeTypes);
            String result = null;
            try {
                result = verify(new String[] { new String().valueOf(i * 100) });
            } catch (MoaException me) {
                me.printStackTrace();
                fail("MoaException");
            }
            switch (i) {
            case 1:
                assertEquals(result, "test1");
                break;
            case 2:
                assertEquals(result, "test2");
                break;
            case 3:
                assertEquals(result, "test3");
                break;
            case 4:
                assertEquals(result, "test4");
                break;
            case 5:
                //
                assertEquals(result, "test5");
                break;
            }
        }
    }

    public void testGetZuseError404() {
        zre = new ZkopfRequest(inputMimeTypes);
        String result = null;
        try {
            result = verify(new String[] { "600" });
        } catch (MoaException me) {
            assertEquals(me.getMessage(), "404 Person unaddressable - no match found");
        }
    }

    public void testGetZuseInfoWrongzbPK() {
        zre = new ZkopfRequest(inputMimeTypes);
        try {
            String result = verify(new String[] { "100" });
            result = verify(new String[] { "700" });
        } catch (MoaException me) {
            assertEquals(me.getMessage(), "403 Person unaddressable - too many hits");
        }
    }

    public void testGetZuseError401() {
        zre = new ZkopfRequest(inputMimeTypes);
        String result = null;
        try {
            result = verify(new String[] { "800" });
            Zuseserver zs = zre.getZs();
            assertEquals(zs, null);
        } catch (MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        }
    }

    public void testGetZuseError410() {
        zre = new ZkopfRequest(inputMimeTypes);
        String result = null;
        try {
            result = verify(new String[] { "810" });
            Zuseserver zs = zre.getZs();
            assertEquals(zs, null);
        } catch (MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        }
    }

    public void testGetZuseError500() {
        zre = new ZkopfRequest(inputMimeTypes);
        String result = null;
        try {
            result = verify(new String[] { "820" });
            Zuseserver zs = zre.getZs();
            assertEquals(zs, null);
        } catch (MoaException me) {
            me.printStackTrace();
            fail("MoaException");
        }
    }

    public void testGetZuseError403() {
        zre = new ZkopfRequest(inputMimeTypes);
        try {
            String result = verify(new String[] { "830" });
        } catch (MoaException me) {
            assertEquals(me.getMessage(), "403 Person unaddressable - too many hits");
        }
    }

    public void testGetZuseInfowithError() {
        zre = new ZkopfRequest(inputMimeTypes);
        //error 404 MimeType or ZuseURLid missing in response
        for (int i = 0; i < 2; i++) {
            String result = null;
            try {
                result = verify(new String[] { new String().valueOf(900 + (i * 10)) });
            } catch (MoaException me) {
                assertEquals(me.getMessage(), "404 Person unaddressable - no match found");
            }
        }
    }

    private String verify(String[] searchValues) throws MoaException {
        zre.queryPerson(searchValues[0]);
        Zuseserver zsv = zre.getZs();
        if (zsv == null)
            return null;
        return zsv.getUrl();
    }
}